Stressant - stress testing tool
===============================

Stressant is a simple yet complete stress-testing tool that forces
a computer to perform a series of test using well-known Linux software
in order to detect possible design or construction failures.

<!-- markdown-toc start - Don't edit this section. Run M-x markdown-toc-generate-toc again -->
**Table of Contents**

 - [Installation](#installation)
     - [Building the image](#building-the-image)
     - [Installing the image](#installing-the-image)
     - [Booting the image using PXE](#booting-the-image-using-pxe)
     - [Testing the image in qemu](#testing-the-image-in-qemu)
 - [Design](#design)
     - [The stressant tool](#the-stressant-tool)
     - [Background](#background)
     - [Remaining work](#remaining-work)
     - [Build system overhaul](#build-system-overhaul)
         - [FAI](#fai)
         - [vmdebootstrap](#vmdebootstrap)
         - [live-build and live-wrapper](#live-build-and-live-wrapper)
         - [Grml](#grml)
 - [References](#references)
 - [Similar software](#similar-software)
 - [Acknowledgements](#acknowledgements)

<!-- markdown-toc end -->

Installation
------------

Stressant is a Python program that can be installed in the usual way:

    pip install stressant

It can also be installed from source:

    ./setup.py install

Stressant is also designed to boot from a live CD or USB stick,
although it may be bootable off the network with a proper
configuration.

Stressant is built on the `vmdebootstrap` tool, which is used to build
the Debian official Live images. You should use at least version 1.4
to build stressant. This of course requires a Debian or Ubuntu
install.

Stressant requires around 1GB of disk space to build an image. The
resulting image is also 1GB with about 650MB in use.

### Building the image

> Instructions are incomplete here.
>
> We are rebuilding this with [Grml][] which changes the way images
> are built. For now, try: grml-live -s stretch -a amd64 This will not
> ship the protype just yet, unfortunately.

### Installing the image

> Note: this should be moved in the main Grml documentation.

You can deploy the ISO on a USB stick with the following command on
most UNIX systems, where `/dev/sdX` is the device for your USB key:

    sudo cp build/stressant-stretch-amd64.iso /dev/sdX

To get better progress information, you can use the fantastic `pv` package:

    pv build/stressant-stretch-amd64.iso | sudo dd of=/dev/sdX conv=fdatasync

For other operating systems, you may try those different options:

 * For *Debian*: see the [official Debian documentation][]
 * For *Ubuntu*: try the [Ubuntu Startup Disk Creator][]
 * For *Fedora* and derivatives: try the [Media Writer][] or follow
   [those instructions][]
 * For *Microsoft Windows*: use the [win32diskimager][] software
 * For all platforms: maybe also try [Etcher][] if the above is too
   complicated

> Note: it seems that Mac OS is constantly changing how this
> works. There used to be support for burning ISO images directly in
> the Disk Utility, but that feature was removed. You may want to try
> [this approach][] if Etcher doesn't work for you.

[Ubuntu Startup Disk creator]: https://www.ubuntu.com/download/desktop/create-a-usb-stick-on-ubuntu
[Media Writer]: https://github.com/MartinBriza/MediaWriter
[those instructions]: https://fedoraproject.org/wiki/How_to_create_and_use_Live_USB
[official Debian documentation]: http://www.debian.org/releases/stable/amd64/ch04s03.html#usb-copy-isohybrid
[this approach]: http://apple.stackexchange.com/a/100458
[Etcher]: https://etcher.io/
[win32diskimager]: https://sourceforge.net/projects/win32diskimager/

### Booting the image using PXE

Inside the ISO file, Grml is build as a [squashfs][] and one can use
the `fetch=URL` argument to fetch that squashfs over the
network. Those [german instructions][] show how to extract the
squashfs from the image and set it up in a PXE menu, but basically,
what you need is:

    kernel grml/2008.11/small-linux26
    append initrd=grml/2008.11/small-minirt26.gz boot=live fetch=http://grml.example.com/grml/grml-small.squashfs

The `small-*` kernel and initrd come from the [grml-terminalserver][]
project.

[german instructions]: http://www.pro-linux.de/kurztipps/2/1432/grml-small-200811-via-pxe-booten-ohne-nfs.html
[squashfs]: https://en.wikipedia.org/wiki/SquashFS

Netbooting images is very useful in all sorts of situations. While
Grml's [grml-terminalserver][] tool helps in automatically configure a
PXE server, you may want to configure your own, more complete
setup. See for now the [Koumbit wiki][PxeMaintenance] for PXE
configuration documentation.

[PxeMaintenance]: https://wiki.koumbit.net/PxeMaintenance

> I have done xperimentes to [boot ISO images over the network][] but
> those have been generally unsuccessful.

[boot ISO images over the network]: http://superuser.com/questions/304722/is-it-possible-to-boot-from-an-iso-over-the-network-and-how

### Testing the image in qemu

This will load the 64 bit image in a KVM-enabled 512 MB image running
only on the serial console.

    qemu-system-x86_64 -m 512 -enable-kvm \
        -serial mon:stdio -display none \
        build/grml96-full_2014-11.iso

Design
------

Stressant is built on top of the [Grml][] project, so it benefits from
its extensive [list of utilities][], which cover most of the rescue
systems out there (e.g. Debian Live and [Debirf][], see below for a
more thorough comparison).

[list of utilities]: http://grml.org/files/#debian
[packaging effort]: http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=707178

Stressant is therefore an ISO image that can be burned on CD/DVD or
copied to a USB drive, or net-bootable images. As part of Grml, there
are already options to perform:

 * memory tests with `memtest86`
 * hardware detection and inventory with [HDT][]

There are also many more options, for example loading to RAM or
setting the system to be read-only, see the [cheatcodes][] list for
more details.

> Stressant does *not* currently ship with Grml, see below for
> details.

[cheatcodes]: http://git.grml.org/?p=grml-live.git;a=blob_plain;f=templates/GRML/grml-cheatcodes.txt;hb=HEAD

[Pure blend]: https://wiki.debian.org/DebianPureBlends
[Debian derivative]: https://wiki.debian.org/Derivatives
[HDT]: http://hdt-project.org/

### The stressant tool

Stressant is therefore mainly a Python program that calls other
UNIX utilities, collects their output on the screen, in a logfile
and/or sends it over email.

The objective of this software is to automate a basic stress-testing
suite that, once started, will go through a basic
CPU/memory/disk/network test framework and report any errors and
failures.

This is done through the `stressant.py` script, which performs the
following tests:

* `lshw` and `smartctl` for hardware inventory
* `dd`, `hdparm`, `fio` and `smartctl` for disk testing - `fio` can
  also overwrite disk drives with the proper options (`--overwrite`
  and `--size=100%`)
* `stress-ng` for CPU testing
* `iperf3` for network testing

Here is an example test run:

    $ sudo ./stressant.py --email anarcat@anarc.at --fileSize 1M --cpuBurnTime 1s --iperfTime 1
    INFO: Starting tests
    INFO: CPU cores: 4
    INFO: Memory: 16 GiB (16715816960 bytes)
    INFO: Hardware inventory
    DEBUG: Calling lshw -short
    OUTPUT: H/W path             Device     Class          Description
    OUTPUT: ==========================================================
    OUTPUT: system         Desktop Computer
    OUTPUT: /0                              bus            NUC6i3SYB
    OUTPUT: /0/0                            memory         64KiB BIOS
    OUTPUT: /0/22                           memory         64KiB L1 cache
    OUTPUT: /0/23                           memory         64KiB L1 cache
    OUTPUT: /0/24                           memory         512KiB L2 cache
    OUTPUT: /0/25                           memory         3MiB L3 cache
    OUTPUT: /0/26                           processor      Intel(R) Core(TM) i3-6100U CPU @ 2.30GHz
    OUTPUT: /0/27                           memory         16GiB System Memory
    OUTPUT: /0/27/0                         memory         16GiB SODIMM DDR4 Synchronous 2133 MHz (0.5 ns)
    OUTPUT: /0/27/1                         memory         [empty]
    OUTPUT: /0/100                          bridge         Skylake Host Bridge/DRAM Registers
    OUTPUT: /0/100/2                        display        HD Graphics 520
    OUTPUT: /0/100/14                       bus            Sunrise Point-LP USB 3.0 xHCI Controller
    OUTPUT: /0/100/14/0          usb1       bus            xHCI Host Controller
    OUTPUT: /0/100/14/0/1        scsi3      storage        USB to ATA/ATAPI Bridge
    OUTPUT: /0/100/14/0/1/0.0.0  /dev/sdb   disk           500GB 00ABYS-01TNA0
    OUTPUT: /0/100/14/0/1/0.0.1  /dev/sdc   disk           500GB 00ABYS-01TNA0
    OUTPUT: /0/100/14/0/3                   input          Dell USB Keyboard
    OUTPUT: /0/100/14/0/4                   input          Kensington Expert Mouse
    OUTPUT: /0/100/14/0/7                   communication  Bluetooth wireless interface
    OUTPUT: /0/100/14/1          usb2       bus            xHCI Host Controller
    OUTPUT: /0/100/14.2                     generic        Sunrise Point-LP Thermal subsystem
    OUTPUT: /0/100/16                       communication  Sunrise Point-LP CSME HECI #1
    OUTPUT: /0/100/17                       storage        Sunrise Point-LP SATA Controller [AHCI mode]
    OUTPUT: /0/100/1c                       bridge         Sunrise Point-LP PCI Express Root Port #5
    OUTPUT: /0/100/1c/0                     network        Wireless 8260
    OUTPUT: /0/100/1e                       generic        Sunrise Point-LP Serial IO UART Controller #0
    OUTPUT: /0/100/1e.6                     generic        Sunrise Point-LP Secure Digital IO Controller
    OUTPUT: /0/100/1f                       bridge         Sunrise Point-LP LPC Controller
    OUTPUT: /0/100/1f.2                     memory         Memory controller
    OUTPUT: /0/100/1f.3                     multimedia     Sunrise Point-LP HD Audio
    OUTPUT: /0/100/1f.4                     bus            Sunrise Point-LP SMBus
    OUTPUT: /0/100/1f.6          eno1       network        Ethernet Connection I219-V
    OUTPUT: /0/1                 scsi2      storage
    OUTPUT: /0/1/0.0.0           /dev/sda   disk           500GB WDC WDS500G1B0B-
    OUTPUT: /0/1/0.0.0/1         /dev/sda1  volume         511MiB Windows FAT volume
    OUTPUT: /0/1/0.0.0/2         /dev/sda2  volume         244MiB EFI partition
    OUTPUT: /0/1/0.0.0/3         /dev/sda3  volume         465GiB EFI partition
    INFO: SMART information for /dev/sda
    DEBUG: Calling smartctl -i /dev/sda
    OUTPUT: smartctl 6.6 2016-05-31 r4324 [x86_64-linux-4.9.0-1-amd64] (local build)
    OUTPUT: Copyright (C) 2002-16, Bruce Allen, Christian Franke, www.smartmontools.org
    OUTPUT: 
    OUTPUT: === START OF INFORMATION SECTION ===
    OUTPUT: Device Model:     WDC WDS500G1B0B-00AS40
    OUTPUT: Serial Number:    XXXXXXXXXXXX
    OUTPUT: LU WWN Device Id: XXXXXXXXXXXX
    OUTPUT: Firmware Version: XXXXXXXXXXXX
    OUTPUT: User Capacity:    500,107,862,016 bytes [500 GB]
    OUTPUT: Sector Size:      512 bytes logical/physical
    OUTPUT: Rotation Rate:    Solid State Device
    OUTPUT: Form Factor:      M.2
    OUTPUT: Device is:        Not in smartctl database [for details use: -P showall]
    OUTPUT: ATA Version is:   ACS-2 T13/2015-D revision 3
    OUTPUT: SATA Version is:  SATA 3.2, 6.0 Gb/s (current: 6.0 Gb/s)
    OUTPUT: Local Time is:    Fri Mar 17 10:24:52 2017 EDT
    OUTPUT: SMART support is: Available - device has SMART capability.
    OUTPUT: SMART support is: Enabled
    OUTPUT: 
    INFO: Basic disk bandwidth tests
    INFO: Writing 1MB file
    DEBUG: Calling dd bs=1M count=512 conv=fdatasync if=/dev/zero of=test
    OUTPUT: 512+0 records in
    OUTPUT: 512+0 records out
    OUTPUT: 536870912 bytes (537 MB, 512 MiB) copied, 1.39591 s, 385 MB/s
    INFO: Reading 1MB file
    DEBUG: Calling dd bs=1M count=512 of=/dev/null if=test
    OUTPUT: 512+0 records in
    OUTPUT: 512+0 records out
    OUTPUT: 536870912 bytes (537 MB, 512 MiB) copied, 0.0848588 s, 6.3 GB/s
    INFO: Hdparm test
    DEBUG: Calling hdparm -Tt /dev/sda
    OUTPUT: 
    OUTPUT: /dev/sda:
    OUTPUT: Timing cached reads:   12406 MB in  2.00 seconds = 6207.39 MB/sec
    OUTPUT: Timing buffered disk reads: 1504 MB in  3.00 seconds = 501.13 MB/sec
    INFO: Disk stress test
    DEBUG: Calling fio --name=stressant --readwrite=randrw --numjob=4 --sync=1 --direct=1 --group_reporting --size=1M --output=/tmp/tmpo2QJnR
    OUTPUT: stressant: (g=0): rw=randrw, bs=4K-4K/4K-4K/4K-4K, ioengine=psync, iodepth=1
    OUTPUT: ...
    OUTPUT: fio-2.16
    OUTPUT: Starting 4 processes
    OUTPUT: 
    OUTPUT: stressant: (groupid=0, jobs=4): err= 0: pid=978: Fri Mar 17 10:25:07 2017
    OUTPUT:   read : io=2160.0KB, bw=5669.3KB/s, iops=1417, runt=   381msec
    OUTPUT:     clat (usec): min=141, max=2197, avg=486.59, stdev=344.70
    OUTPUT:      lat (usec): min=141, max=2198, avg=486.96, stdev=344.71
    OUTPUT:     clat percentiles (usec):
    OUTPUT:      |  1.00th=[  145],  5.00th=[  153], 10.00th=[  161], 20.00th=[  175],
    OUTPUT:      | 30.00th=[  189], 40.00th=[  217], 50.00th=[  278], 60.00th=[  676],
    OUTPUT:      | 70.00th=[  748], 80.00th=[  828], 90.00th=[  916], 95.00th=[  980],
    OUTPUT:      | 99.00th=[ 1384], 99.50th=[ 1800], 99.90th=[ 2192], 99.95th=[ 2192],
    OUTPUT:      | 99.99th=[ 2192]
    OUTPUT:   write: io=1936.0KB, bw=5081.4KB/s, iops=1270, runt=   381msec
    OUTPUT:     clat (usec): min=618, max=6602, avg=2566.13, stdev=1029.39
    OUTPUT:      lat (usec): min=619, max=6602, avg=2566.71, stdev=1029.39
    OUTPUT:     clat percentiles (usec):
    OUTPUT:      |  1.00th=[  732],  5.00th=[  900], 10.00th=[  964], 20.00th=[ 1672],
    OUTPUT:      | 30.00th=[ 1976], 40.00th=[ 2384], 50.00th=[ 2640], 60.00th=[ 3152],
    OUTPUT:      | 70.00th=[ 3312], 80.00th=[ 3440], 90.00th=[ 3568], 95.00th=[ 3856],
    OUTPUT:      | 99.00th=[ 4704], 99.50th=[ 4960], 99.90th=[ 6624], 99.95th=[ 6624],
    OUTPUT:      | 99.99th=[ 6624]
    OUTPUT:     lat (usec) : 250=24.41%, 500=4.88%, 750=7.91%, 1000=19.34%
    OUTPUT:     lat (msec) : 2=10.84%, 4=30.66%, 10=1.95%
    OUTPUT:   cpu          : usr=0.80%, sys=2.39%, ctx=1945, majf=0, minf=35
    OUTPUT:   IO depths    : 1=100.0%, 2=0.0%, 4=0.0%, 8=0.0%, 16=0.0%, 32=0.0%, >=64=0.0%
    OUTPUT:      submit    : 0=0.0%, 4=100.0%, 8=0.0%, 16=0.0%, 32=0.0%, 64=0.0%, >=64=0.0%
    OUTPUT:      complete  : 0=0.0%, 4=100.0%, 8=0.0%, 16=0.0%, 32=0.0%, 64=0.0%, >=64=0.0%
    OUTPUT:      issued    : total=r=540/w=484/d=0, short=r=0/w=0/d=0, drop=r=0/w=0/d=0
    OUTPUT:      latency   : target=0, window=0, percentile=100.00%, depth=1
    OUTPUT: 
    OUTPUT: Run status group 0 (all jobs):
    OUTPUT:    READ: io=2160KB, aggrb=5669KB/s, minb=5669KB/s, maxb=5669KB/s, mint=381msec, maxt=381msec
    OUTPUT:   WRITE: io=1936KB, aggrb=5081KB/s, minb=5081KB/s, maxb=5081KB/s, mint=381msec, maxt=381msec
    OUTPUT: 
    OUTPUT: Disk stats (read/write):
    OUTPUT:     dm-3: ios=207/493, merge=0/0, ticks=120/296, in_queue=416, util=58.78%, aggrios=540/1527, aggrmerge=0/0, aggrticks=288/752, aggrin_queue=1040, aggrutil=75.30%
    OUTPUT:     dm-0: ios=540/1527, merge=0/0, ticks=288/752, in_queue=1040, util=75.30%, aggrios=540/1326, aggrmerge=0/201, aggrticks=264/704, aggrin_queue=968, aggrutil=74.49%
    OUTPUT:   sda: ios=540/1326, merge=0/201, ticks=264/704, in_queue=968, util=74.49%
    INFO: CPU stress test for 1s
    DEBUG: Calling stress-ng --timeout 1s --cpu 0 --ignite-cpu --metrics-brief --log-brief --tz --times --aggressive
    OUTPUT: dispatching hogs: 4 cpu
    OUTPUT: cache allocate: default cache size: 3072K
    OUTPUT: successful run completed in 1.05s
    OUTPUT: stressor       bogo ops real time  usr time  sys time   bogo ops/s   bogo ops/s
    OUTPUT: (secs)    (secs)    (secs)   (real time) (usr+sys time)
    OUTPUT: cpu                 453      1.04      2.80      0.00       437.62       161.79
    OUTPUT: cpu:
    OUTPUT: acpitz   27.80 °C
    OUTPUT: pch_skylake   32.77 °C
    OUTPUT: acpitz   31.78 °C
    OUTPUT: x86_pkg_temp   34.40 °C
    OUTPUT: for a 1.05s run time:
    OUTPUT: 4.22s available CPU time
    OUTPUT: 2.81s user time   ( 66.64%)
    OUTPUT: 0.01s system time (  0.24%)
    OUTPUT: 2.82s total time  ( 66.87%)
    OUTPUT: load average: 0.34 0.58 2.52
    INFO: Running network benchmark
    DEBUG: Calling iperf3 -c iperf.he.net -t 1
    OUTPUT: iperf3: error - the server is busy running a test. try again later
    ERROR: Command failed: Command 'iperf3 -c iperf.he.net -t 1' returned non-zero exit status 1
    INFO: all done
    INFO: sent email to ['anarcat@anarc.at'] using anarc.at


We currently use the `iperf.he.net` server from [Hurricane Electric][]
as a default server for our tests, but users are encouraged to change
that to a local server using the `--iperfServer` argument to get more
accurate results.

[Hurricane Electric]: https://he.net/

### Background

This project emanates from a [packaging effort][] of a custom Linux
distribution called 'breakin'. It turned into a simple Python program
that reuses existing stress-testing programs packaged in Debian.

Stressant *used* to be built as a standalone Debian Derivative, a
[Pure blend][] based on [Debirf][]. But in 2017, the project was
rearchitectured to be based on Grml and to focus on developing a
standalone stress-testing tool. While Stressant could still become its
own [Debian derivative][], it is the hope of the author that it will be
factored into Grml and be part of the project without needing to
create the heavy infrastructure for another Linux distribution.

The [Is your Computer Stable?][] post from Jeff Atwood was a
motivation to get back into the project. It outlines a few basic tools
to use to make sure your computer is stable:

 * memtest86 - shipped with Grml
 * install Ubuntu - we assume you'll do that anyways
 * [MPrime][] to stress the CPU - not free software, [stress-ng][] was
   chosen instead and gives similar results here
 * [badblocks](https://en.wikipedia.org/wiki/Badblocks) test (with
   `-sv`) - this is covered by the `fio` test
 * smartctl -i/-a/-t to identify and test harddrives
 * `dd` and `hdparm` to get quick stats - done
 * [bonnie++][] for more extensive benchmarks - Grml people suggested we
   use [fio][] instead
 * [iperf][] for network testing - this assumes a local server, we
   instead use [iperf3][] and [public servers][]
 * [furmark][] for testing the GPU - Windows-only, no Linux
   equivalent, the [Phoronix test suite][] uses [ffmpeg tests][] for
   that purpose

[MPrime]: http://www.mersenne.org/download/
[iperf]: https://en.wikipedia.org/wiki/Iperf
[iperf3]: http://software.es.net/iperf/
[public servers]: https://iperf.fr/iperf-servers.php
[bonnie++]: https://en.wikipedia.org/wiki/Bonnie%2B%2B
[fio]: http://fio.readthedocs.org/
[furmark]: http://www.ozone3d.net/benchmarks/fur/
[ffmpeg tests]: https://openbenchmarking.org/test/pts/ffmpeg

The idea was to regroup this in a single tool that would perform all
those tests, without reinventing the wheel of course.

Stressant was also highly coupled with [Koumbit][]'s infrastructure as
this is where the Debirf recipes were originally developed. It needed
a CI system to build the images, which was originally done with
Jenkins. This was then done with Gitlab CI, but failed to build images
because of issues with debirf and, ultimately, docker itself. This is
why why Grml was used as a basis for future development.

[Koumbit]: https://koumbit.org

### Remaining work

Stressant could run in a tmux or screen session that would show the
current task on one pane and syslog (or journalctl) in another. This
would allow for more information to be crammed in a single display
while at the same time making remote access (e.g. through SSH) easier
to switch to.

Finally, we need clear and better documentation on various testing
tools there are out there, a bit like TAILS is doing. For example, we
used to ship with `diskscan` but i didn't even remember that and I am
not sure what to use it for or when to use it. A summary description
of the available tools, maybe through a menu system or at least a set
of HTML files, would be useful. I would probably use Sphinx and RST
for this because of the simplicity and availability of tools like
readthedocs.org and the ease of creationg for offline documentation
(PDF and ePUB).

### Build system overhaul

Debirf was initially as a buildsystem chosen because it was simple and easy to
modify. We are considering other tools for generating images because
debirf is too limited for our needs: it doesn't provide a way to embed
boot-level, arbitrary binaries like memtest86 because we depend on the
Linunx kernel. Furthermore, we are currently having serious issues
building any debirf image, either in Debian 8 ([bug #806377][]) or 9
([bug #848834][]).

[bug #848834]: http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=848834
[bug #806377]: https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=806377

There are a lot of options to build images. The [Debian cloud team][]
considered a few tools to generate their cloud images, and a few are
relevant here ([FAI][] and [vmdebootstrap][]), see [this post][] for
details. Here's an outline of the possibilities.

 * [FAI][]'s [fai-diskimage][]: requires root and is designed for
   cloud images, not ISO, see below
 * [vmdebootstrap][]: minimal, written in Python, used for [Debian live][]
   images, requires root (for loop filesystems creation), doesn't
   support shipping the Debian installer anymore?
 * [live-build][]: tools used by [Debian live][] and other blends
   (e.g. [PGP cleanroom][], see how it [uses live-build][]), now uses
   [live-wrapper][] which uses [vmdebootstrap][]
 * [grml-debootstrap][]: similar to vmdebootstrap, not tested in
   details
 * there's also this [exhaustive list of tools][] to build Debian
   systems

[Debian cloud team]: https://wiki.debian.org/Teams/Cloud
[FAI]: https://wiki.debian.org/FAI
[vmdebootstrap]: https://vmdebootstrap.alioth.debian.org/
[grml-debootstrap]: http://grml.org/grml-debootstrap/
[this post]: https://lists.debian.org/debian-cloud/2016/11/msg00100.html
[live-build]: https://tracker.debian.org/pkg/live-build
[fixed in FAI 5.3.3]: https://lists.debian.org/debian-cloud/2017/01/msg00003.html
[exhaustive list of tools]: https://wiki.debian.org/SystemBuildTools
[Debian live]: https://www.debian.org/devel/debian-live/
[cleartext URL]: http://fai-project.org/download/basefiles/
[fai-diskimage]: http://fai-project.org/doc/man/fai-diskimage.html
[live-wrapper]: https://live-wrapper.readthedocs.io/en/latest/
[uses live-build]: https://anonscm.debian.org/cgit/collab-maint/make-pgp-clean-room.git/tree/scripts
[PGP cleanroom]: https://wiki.debian.org/OpenPGP/CleanRoomLiveEnvironment

#### FAI

I have tried to use FAI to follow the lead of the
[Debian cloud team][]. Unforunately, I stumbled upon a few
bugs. First, `fai-diskimage` would fail to build an image if the host
uses LVM. This was [fixed in FAI 5.3.3][] (or maybe 5.3.4?). Also, FAI
seems to fetch base files from a [cleartext URL][], which seems like a
dubious security choice.

After finding [this tutorial][], I figured I would give it a try
again. Unfortunately, after asking on IRC (`#debian-cloud` on OFTC), I
was told (by Noah!) that "*fai-diskimage is probably not what you want
for an iso image*" and they suggested I use `fai-cd`. Unfortunately,
`fai-cd` works completely differently: it doesn't support the
`--class` system that `fai-diskimage` was built with, so we can't
reuse those already mysterious recipes. `fai-cd` seems to be built
towards creating install medium and not live images.

All this seems to make FAI mostly unusable for the task at hand,
although it *should* be noted that [grml-live][] uses FAI to build
their images...

[grml-live]: http://grml.org/grml-live/

[this tutorial]: https://noah.meyerhans.us/blog/2017/02/10/using-fai-to-customize-and-build-your-own-cloud-images/

#### vmdebootstrap

So far we have had good results with `vmdebootstrap`, but the fact
that it requires a loop device has made it difficult to use Gitlab's
CI system. Docker has a [bug][] that makes it impossible to use loop
devices and `kpartx` commands in it. So to build images through
Gitlab's CI would require full virtualization instead of just Docker,
something that's not provided by Gitlab.com right now. Worse:
VirtualBox may not [make it to stretch at all][] which makes it
difficult to deploy new builders for it. This problem is probably
shared by all image building tools, however, so it's unclear if there
would be a benefit to switching to another tool right now.

[make it to stretch at all]: https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=794466
[bug]: https://github.com/docker/docker/issues/27886

#### live-build and live-wrapper

It is unclear if I am better off using live build or vmdebootstrap
directly. The PGP cleanroom build uses live build, so maybe I should
do that as well...

Notes: live-wrapper is where the idea of [using HDT][] comes
from. Unfortunately, it looks like the boot menus don't actually work
yet ([bug #813527][]). Furthermore, live-wrapper doesn't support
[initrd-style netboot][bug #849015], so we would need documentation on
how to boot from ISO files over PXE.

[bug #849015]: https://bugs.debian.org/849015
[bug #813527]: https://bugs.debian.org/813527
[using HDT]: https://anonscm.debian.org/cgit/debian-live/live-wrapper.git/tree/lwr/run.py

#### Grml

[Grml][] is a project quite similar to stressant:

 * based on Debian
 * provides rescue tools
 * [live CD/USB image](http://grml.org/grml-live/)
 * also provides support for netboot through [grml-terminalserver][],
   which can [use remote squashfs][]

The project is really interesting and maybe we should focus on
creating an integrated stress-test tool on *top* of grml instead of
trying to fix all the issues those guys are already struggling with...

It has most of the packages we need, except those:

    blktool
    bonnie++
    chntpw
    diskscan
    e2tools
    fatresize
    foremost
    hfsplus
    i7z
    lm-sensors
    mtd-utils
    scrub
    smp-utils
    stress-ng
    tofrodos
    u-boot-tools
    wodim

Some of those are actually required by our prototype.

So remaining issues with Grml integration:

 1. how to ship a non-official Debian package in Grml? we can
    [add official packages][], but not directly a `.deb`... we could
    also [include files directly][], but then we have the problem of
    Python dependencies... we can use the `debs` boot option to
    install `.deb` files from `/` in the image at least.
 2. review the above packages we collected from various rescue modes
    and see if they are relevant
 3. hook stressant in the magic Grml menu to start directly from the
    boot menu - we can use the `scripts=path-name` argument for this,
    it looks in the "`DCS` dir", which is `/` or whatever `myconfig=`
    points at
 4. figure out how to [chain into memtest86][] to complete the test
    suite

[chain into memtest86]: https://github.com/grml/live-boot-grml/issues/5
[include files directly]: http://grml.org/grml-live/#install-local-files
[add official packages]: http://grml.org/grml-live/#add-additional-debian-packages
[grml-terminalserver]: http://grml.org/terminalserver/
[Grml]: http://grml.org/
[use remote squashfs]: http://www.pro-linux.de/kurztipps/2/1432/grml-small-200811-via-pxe-booten-ohne-nfs.html

References
----------

 * [Home page][]
 * [Debirf][]
 * [Qemu][]
 * [Qemu cheat sheet at Koumbit][]
 * [Is your Computer Stable?][] - another inspiration
 * [OpenBenchmarking.org][] - a good source of benchmarking tools

[Home page]: https://gitlab.com/anarcat/stressant
[Debirf]: http://cmrg.fifthhorseman.net/wiki/debirf
[Qemu]: http://qemu.org/
[Qemu cheat sheet at Koumbit]: https://wiki.koumbit.net/Qemu
[OpenBenchmarking.org]: https://openbenchmarking.org/
[Is your Computer Stable?]: https://blog.codinghorror.com/is-your-computer-stable/

Similar software
----------------

* [Phoronix test suite][] - far-ranging benchmarking suite
* [Inquisitor][] - hardware testing suite
* [Breakin][] - stress-test and hardware diagnostics tool
* [FWTS][] - Ubuntu's Firmware Test Suite - performs sanity checks on
  Intel/AMD PC firmware. It is intended to identify BIOS and ACPI
  errors and if appropriate it will try to explain the errors and give
  advice to help workaround or fix firmware bugs
* [chipsec][] - framework for analyzing the security of PC platforms
  including hardware, system firmware (BIOS/UEFI), and platform
  components
* [sys_basher][] - another stress-testing tool
* [PerfKit Benchmarker][] - GCP's benchmarking tool

[Breakin]: http://www.advancedclustering.com/products/software/breakin/
[FWTS]: https://wiki.ubuntu.com/FirmwareTestSuite/
[chipsec]: https://github.com/chipsec/chipsec
[Inquisitor]: https://en.wikipedia.org/wiki/Inquisitor_(hardware_testing_software)
[Phoronix test suite]: https://en.wikipedia.org/wiki/Phoronix_Test_Suite
[sys_basher]: http://www.polybus.com/sys_basher_web/
[PerfKit Benchmarker]: https://github.com/GoogleCloudPlatform/PerfKitBenchmarker

Acknowledgements
----------------

Thanks to jrollins & dkg: for writing debirf, and especially to dkg
for help with qemu and other oddities. Thanks also to taggart for
spurring this idea forward.
